@@ -107,7 +107,7 @@ DownloadStore::save(Download* d) {
 
   // Move this somewhere else?
   d->bencode()->get_key("rtorrent").insert_key("total_uploaded", d->download()->up_rate()->total());
-  d->bencode()->get_key("rtorrent").insert_key("chunks_done", d->download()->chunks_done());
+  d->bencode()->get_key("rtorrent").insert_key("chunks_done", d->download()->file_list()->completed_chunks());
 
   torrent::Object& resumeObject = d->download()->bencode()->get_key("libtorrent_resume");
 
@@ -174,7 +174,7 @@ DownloadStore::is_correct_format(std::string f) {
 
 std::string
 DownloadStore::create_filename(Download* d) {
-  return m_path + rak::transform_hex(d->download()->info_hash()) + ".torrent";
+  return m_path + rak::transform_hex(d->download()->info_hash().begin(), d->download()->info_hash().end()) + ".torrent";
 }
 
 }
