@@ -49,6 +49,7 @@ namespace display {
 
 Manager::Manager() :
   m_forceRedraw(false) {
+
   m_taskUpdate.set_slot(rak::mem_fn(this, &Manager::receive_update));
 }
 
@@ -61,79 +62,25 @@ Manager::force_redraw() {
   m_forceRedraw = true;
 }
 
-
-Manager::iterator
-Manager::insert(iterator pos, Window* w) {
-  return Base::insert(pos, w);
-}
-
-// Swap with the function below.
-Manager::iterator
-Manager::erase(iterator pos) {
-  if (pos != end())
-    return erase(*pos);
-  else
-    return end();
-}
-
-Manager::iterator
-Manager::erase(Window* w) {
-  iterator itr = std::find(begin(), end(), w);
-
-  if (itr == end())
-    throw std::logic_error("Manager::erase(...) did not find the window");
-
-  return Base::erase(itr);
-}
-
-Manager::iterator
-Manager::find(Window* w) {
-  return std::find(begin(), end(), w);
-}
-
 void
 Manager::schedule(Window* w, rak::timer t) {
   rak::priority_queue_erase(&m_scheduler, w->task_update());
   rak::priority_queue_insert(&m_scheduler, w->task_update(), t);
-  schedule_update();
+  schedule_update(50000);
 }
 
 void
 Manager::unschedule(Window* w) {
   rak::priority_queue_erase(&m_scheduler, w->task_update());
-  schedule_update();
+  schedule_update(50000);
 }
 
 void
 Manager::adjust_layout() {
-  int staticHeight = std::for_each(begin(), end(),
-				   rak::if_then(std::mem_fun(&Window::is_active),
-						rak::accumulate(0, std::mem_fun(&Window::get_min_height)))).m_then.result;
-  int countDynamic = std::for_each(begin(), end(),
-				   rak::if_then(std::mem_fun(&Window::is_active),
-						rak::accumulate(0, std::mem_fun(&Window::is_dynamic)))).m_then.result;
+  Canvas::redraw_std();
+  m_rootFrame.balance(0, 0, Canvas::get_screen_width(), Canvas::get_screen_height());
 
-  int dynamic = std::max(0, Canvas::get_screen_height() - staticHeight);
-  int height = 0, h;
-
-  for (iterator itr = begin(); itr != end(); ++itr, height += h) {
-    h = 0;
-
-    if (!(*itr)->is_active())
-      continue;
-
-    if ((*itr)->is_dynamic()) {
-      dynamic -= h = (dynamic + countDynamic - 1) / countDynamic;
-      countDynamic--;
-    } else {
-      h = 0;
-    }
-
-    h += (*itr)->get_min_height();
-
-    (*itr)->resize(0, height, Canvas::get_screen_width(), h);
-    (*itr)->mark_dirty();
-  }
+  schedule_update(0);
 }
 
 void
@@ -143,22 +90,24 @@ Manager::receive_update() {
 
     display::Canvas::resize_term(display::Canvas::term_size());
     Canvas::redraw_std();
+
     adjust_layout();
+    m_rootFrame.redraw();
   }
 
   Canvas::refresh_std();
 
   rak::priority_queue_perform(&m_scheduler, cachedTime);
-  std::for_each(begin(), end(), rak::if_then(std::mem_fun(&Window::is_active), std::mem_fun(&Window::refresh)));
+  m_rootFrame.refresh();
 
   Canvas::do_update();
 
   m_timeLastUpdate = cachedTime;
-  schedule_update();
+  schedule_update(50000);
 }
 
 void
-Manager::schedule_update() {
+Manager::schedule_update(uint32_t minInterval) {
   if (m_scheduler.empty()) {
     rak::priority_queue_erase(&taskScheduler, &m_taskUpdate);
     return;
@@ -166,8 +115,7 @@ Manager::schedule_update() {
 
   if (!m_taskUpdate.is_queued() || m_taskUpdate.time() > m_scheduler.top()->time()) {
     rak::priority_queue_erase(&taskScheduler, &m_taskUpdate);
-    rak::priority_queue_insert(&taskScheduler, &m_taskUpdate,
-			       std::max(m_scheduler.top()->time(), m_timeLastUpdate + 50000));
+    rak::priority_queue_insert(&taskScheduler, &m_taskUpdate, std::max(m_scheduler.top()->time(), m_timeLastUpdate + minInterval));
   }
 }
 
