@@ -41,19 +41,18 @@
 
 #include "window.h"
 
-class Control;
-
 namespace display {
 
 class WindowStatusbar : public Window {
 public:
-  WindowStatusbar(Control* c);
+  WindowStatusbar() :
+    Window(new Canvas, 0, 0, 1, extent_full, extent_static),
+    m_lastTick(0) {}
 
   virtual void   redraw();
 
 private:
   uint64_t       m_lastTick;
-  Control*       m_control;
 };
 
 }
