@@ -39,12 +39,14 @@
 #include <sigc++/bind.h>
 #include <torrent/exceptions.h>
 #include <torrent/rate.h>
+#include <torrent/hash_string.h>
+#include <torrent/peer/peer_info.h>
 
-#include "display/client_info.h"
 #include "display/frame.h"
 #include "display/manager.h"
 #include "display/text_element_helpers.h"
 #include "display/text_element_lambda.h"
+#include "display/utils.h"
 #include "display/window_peer_list.h"
 #include "input/manager.h"
 
@@ -87,6 +89,14 @@ ElementPeerList::~ElementPeerList() {
   delete m_elementInfo;
 }
 
+std::string
+te_client_version(const torrent::PeerInfo* info) {
+  char buf[128];
+  display::print_client_version(buf, buf + 128, info->client_info());
+
+  return buf;
+}
+
 inline ElementText*
 ElementPeerList::create_info() {
   using namespace display::helpers;
@@ -106,9 +116,11 @@ ElementPeerList::create_info() {
                        display::text_element_value_slot(rak::on(std::mem_fun(&torrent::Peer::address), std::ptr_fun(&te_port))));
 
   element->push_column("Id:",        display::text_element_string_slot(std::mem_fun(&torrent::Peer::id), string_base::flag_escape_html));
-  element->push_column("Client:",    display::text_element_string_slot(rak::on(std::mem_fun(&torrent::Peer::id), rak::make_mem_fun(control->client_info(), &display::ClientInfo::str_str))));
+//   element->push_column("Client:",    display::text_element_string_slot(rak::on(std::mem_fun(&torrent::Peer::id), rak::make_mem_fun(control->client_info(), &display::ClientInfo::hash_str))));
+  element->push_column("Client:",    display::text_element_string_slot(rak::on(std::mem_fun(&torrent::Peer::info), std::ptr_fun(&te_client_version))));
   element->push_column("Options:",   display::text_element_string_slot(std::mem_fun(&torrent::Peer::options), string_base::flag_escape_hex | string_base::flag_fixed_width, 0, 8));
   element->push_column("Connected:", display::text_element_branch(std::mem_fun(&torrent::Peer::is_incoming), te_string("incoming"), te_string("outgoing")));
+  element->push_column("Encrypted:", display::text_element_branch3(std::mem_fun(&torrent::Peer::is_encrypted), te_string("yes"), std::mem_fun(&torrent::Peer::is_obfuscated), te_string("handshake"), te_string("no")));
 
   element->push_back("");
   element->push_column("Snubbed:", display::text_element_branch(std::mem_fun(&torrent::Peer::is_snubbed), te_string("yes"), te_string("no")));
@@ -128,7 +140,7 @@ ElementPeerList::create_info() {
 void
 ElementPeerList::activate(display::Frame* frame, bool focus) {
   if (is_active())
-    throw torrent::client_error("ui::ElementPeerList::activate(...) is_active().");
+    throw torrent::internal_error("ui::ElementPeerList::activate(...) is_active().");
 
   if (focus)
     control->input()->push_back(&m_bindings);
@@ -142,7 +154,7 @@ ElementPeerList::activate(display::Frame* frame, bool focus) {
 void
 ElementPeerList::disable() {
   if (!is_active())
-    throw torrent::client_error("ui::ElementPeerList::disable(...) !is_active().");
+    throw torrent::internal_error("ui::ElementPeerList::disable(...) !is_active().");
 
   control->input()->erase(&m_bindings);
 
@@ -197,7 +209,7 @@ ElementPeerList::receive_next() {
   else
     m_listItr = m_list.begin();
 
-  updated_itr();
+  update_itr();
 }
 
 void
@@ -207,7 +219,7 @@ ElementPeerList::receive_prev() {
   else
     m_listItr = m_list.end();
 
-  updated_itr();
+  update_itr();
 }
 
 void
@@ -228,14 +240,14 @@ ElementPeerList::receive_peer_disconnected(torrent::Peer p) {
   PList::iterator itr = std::find(m_list.begin(), m_list.end(), p);
 
   if (itr == m_list.end())
-    throw torrent::client_error("ElementPeerList::receive_peer_disconnected(...) itr == m_list.end().");
+    throw torrent::internal_error("ElementPeerList::receive_peer_disconnected(...) itr == m_list.end().");
 
   if (itr == m_listItr)
     m_listItr = m_list.erase(itr);
   else
     m_list.erase(itr);
 
-  updated_itr();
+  update_itr();
 }
 
 void
@@ -245,11 +257,11 @@ ElementPeerList::receive_snub_peer() {
 
   m_listItr->set_snubbed(!m_listItr->is_snubbed());
 
-  updated_itr();
+  update_itr();
 }
 
-inline void
-ElementPeerList::updated_itr() {
+void
+ElementPeerList::update_itr() {
   m_windowList->mark_dirty();
   m_elementInfo->set_object(m_listItr != m_list.end() ? &*m_listItr : NULL);
 }
