@@ -64,7 +64,10 @@ Root::Root() :
   m_windowTitle(NULL),
   m_windowHttpQueue(NULL),
   m_windowInput(NULL),
-  m_windowStatusbar(NULL) {
+  m_windowStatusbar(NULL),
+
+  m_maxUploadsGlobal(0),
+  m_maxDownloadsGlobal(0) {
 }
 
 void
@@ -135,6 +138,12 @@ Root::setup_keys() {
     m_bindings['A']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), 1);
     m_bindings['Y']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -1);
 
+  } else if (strcasecmp(control->variable()->get_string("key_layout").c_str(), "dvorak") == 0) {
+    m_bindings['a']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), 1);
+    m_bindings[';']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), -1);
+    m_bindings['A']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), 1);
+    m_bindings[':']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -1);
+
   } else {
     m_bindings['a']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), 1);
     m_bindings['z']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), -1);
@@ -142,15 +151,28 @@ Root::setup_keys() {
     m_bindings['Z']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -1);
   }
 
-  m_bindings['s']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), 5);
-  m_bindings['x']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), -5);
-  m_bindings['S']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), 5);
-  m_bindings['X']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -5);
+  if (strcasecmp(control->variable()->get_string("key_layout").c_str(), "dvorak") != 0) {
+    m_bindings['s']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), 5);
+    m_bindings['x']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), -5);
+    m_bindings['S']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), 5);
+    m_bindings['X']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -5);
 
-  m_bindings['d']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), 50);
-  m_bindings['c']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), -50);
-  m_bindings['D']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), 50);
-  m_bindings['C']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -50);
+    m_bindings['d']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), 50);
+    m_bindings['c']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), -50);
+    m_bindings['D']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), 50);
+    m_bindings['C']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -50);
+
+  } else {
+    m_bindings['o']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), 5);
+    m_bindings['q']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), -5);
+    m_bindings['O']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), 5);
+    m_bindings['Q']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -5);
+
+    m_bindings['e']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), 50);
+    m_bindings['j']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_up_throttle), -50);
+    m_bindings['E']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), 50);
+    m_bindings['J']           = sigc::bind(sigc::mem_fun(*this, &Root::adjust_down_throttle), -50);
+  }
 
   m_bindings['\x0C']        = sigc::mem_fun(m_control->display(), &display::Manager::force_redraw); // ^L
   m_bindings['\x11']        = sigc::mem_fun(m_control, &Control::receive_normal_shutdown); // ^Q
@@ -162,6 +184,27 @@ Root::set_down_throttle(unsigned int throttle) {
     m_windowStatusbar->mark_dirty();
 
   torrent::set_down_throttle(throttle * 1024);
+
+  int64_t div = control->variable()->get_value("max_downloads_div");
+
+  if (throttle == 0 || div <= 0) {
+    torrent::set_max_download_unchoked(m_maxDownloadsGlobal);
+    return;
+  }
+
+  throttle /= control->variable()->get_value("max_downloads_div");
+
+  unsigned int maxUnchoked;
+
+  if (throttle <= 10)
+    maxUnchoked = 1 + throttle / 1;
+  else
+    maxUnchoked = 10 + throttle / 5;
+
+  if (m_maxDownloadsGlobal != 0)
+    torrent::set_max_download_unchoked(std::min(maxUnchoked, m_maxDownloadsGlobal));
+  else
+    torrent::set_max_download_unchoked(maxUnchoked);
 }
 
 void
@@ -171,14 +214,26 @@ Root::set_up_throttle(unsigned int throttle) {
 
   torrent::set_up_throttle(throttle * 1024);
 
-  if (throttle == 0)
-    torrent::set_max_unchoked(0);
+  int64_t div = control->variable()->get_value("max_uploads_div");
 
-  else if (throttle <= 10)
-    torrent::set_max_unchoked(1 + throttle / 1);
+  if (throttle == 0 || div <= 0) {
+    torrent::set_max_unchoked(m_maxUploadsGlobal);
+    return;
+  }
 
+  throttle /= control->variable()->get_value("max_uploads_div");
+
+  unsigned int maxUnchoked;
+
+  if (throttle <= 10)
+    maxUnchoked = 1 + throttle / 1;
   else
-    torrent::set_max_unchoked(10 + throttle / 5);
+    maxUnchoked = 10 + throttle / 5;
+
+  if (m_maxUploadsGlobal != 0)
+    torrent::set_max_unchoked(std::min(maxUnchoked, m_maxUploadsGlobal));
+  else
+    torrent::set_max_unchoked(maxUnchoked);
 }
 
 void
@@ -191,6 +246,26 @@ Root::adjust_up_throttle(int throttle) {
   set_up_throttle(std::max<int>(torrent::up_throttle() / 1024 + throttle, 0));
 }
 
+void
+Root::set_max_uploads_global(int64_t slots) {
+  if (slots < 0)
+    throw torrent::input_error("Out of range.");
+
+  m_maxUploadsGlobal = slots;
+
+  set_up_throttle(torrent::up_throttle() / 1024);
+}
+
+void
+Root::set_max_downloads_global(int64_t slots) {
+  if (slots < 0)
+    throw torrent::input_error("Out of range.");
+
+  m_maxDownloadsGlobal = slots;
+
+  set_down_throttle(torrent::down_throttle() / 1024);
+}
+
 void
 Root::enable_input(const std::string& title, input::TextInput* input) {
   if (m_windowInput->input() != NULL)
